var toolbox      = require('@spark-engine/toolbox'),
    Event        = require('@spark-engine/event'),
    EventManager = Event.callbackManager,
    Modals       = [],
    Stack        = [],
    FocusStack   = []

// CSS styles to help prevent stray modal clicks (doesn't work in less than IE 11)
var style        = '<style id="modal-toggle-style">body[data-active-modal] * { pointer-events: none } .active-modal, .active-modal * { pointer-events: all }</style>'

// types of modals
// Menu - trigger toggles, clicking outside closes
// Panel - trigger opens, but does not close, clicking outside (or esc) closes
// Dialog - trigger opens, clicking outside is ignored, must be closed by clicking a close button or the escape key


var Modal = {
  new: function(el, type, trigger) {

    // Check if a modal is already registered
    // This might happen if there is more than one trigger for a modal
    var m = getModal(el)
    if (m) {
      // Register the trigger and return
      if (trigger) m.triggers.push(trigger)
      return getModal(el)
    }

    var modal = {
      triggers: [],
      _modal:   true,
      type:     type,
      id:       Modals.length,
      el:       el,

      events: {
        open: EventManager.new(),
        close: EventManager.new()
      },

      isOpen: false
    }

    modal[ type ] = true // convenience type check
    Modals.push(modal)

    // Tag modal element with an id
    el.dataset.modalId = modal.id
    // Hide modal by default
    el.classList.add('hidden')
    el.setAttribute('hidden', true)
    el.setAttribute('aria-hidden', true)
    el.setAttribute('aria-modal', true)

    // Register elements which trigger the modal
    if (trigger) modal.triggers.push(trigger)

    modal.on = function (type, callback) {
      if (modal.events[type]) {
        modal.events[type].add(callback)
      }
    }

    // Add default opening behavior
    modal.open = function () {
      if (modal.isOpen) return
      modal.isOpen = true
      modal.events.open.start()
      modal.events.close.stop()

      // Listen to escape key presses in 'modal-active' scope
      if (!Stack.length) Event.on(document, 'keydown', keyPress)

      FocusStack.push(document.activeElement)
      Stack.push(modal)

      document.body.classList.add('modal-active')
      document.body.dataset.modal = el.id || ''

      el.removeAttribute('hidden')
      el.classList.remove('hidden')
      el.setAttribute('aria-hidden', false)

      // Set state on modal triggers
      modal.triggers.forEach(function(el) {
        el.classList.add('modal-active')
      })

      el.classList.remove('modal-is-closing')
      el.classList.add('modal-is-opening')

      // After opening animation, remove animation class
      Event.afterAnimation(el, modal.events.open.fire, 50)
    }

    // Add first open callback
    modal.events.open.add(function () {
      el.classList.remove('modal-is-opening')
    })

    modal.close = function () {
      if (!modal.isOpen) return

      // Toggle event callbacks
      modal.events.open.stop()
      modal.events.close.start()

      el.classList.remove('modal-is-opening')
      el.classList.add('modal-is-closing')

      // After closing animation, hide, pop stack, fire close events.
      Event.afterAnimation(el, modal.events.close.fire, 50)
    }

    // Add first close callback
    modal.events.close.add(function(){
      modal.isOpen = false

      el.classList.remove('modal-is-closing')
      el.classList.add('hidden')
      el.setAttribute('hidden', true)
      el.setAttribute('aria-hidden', true)

      // Set state on modal triggers
      modal.triggers.forEach(function(el) {
        el.classList.remove('modal-active')
      })

      // Remove this modal from the stack
      Stack.pop()

      // Restore foucs for dialogs and modal menus
      // This returns users to their place on the page
      // This doesn't occur on modal panels because focusing
      // opens panels. There may be a good fix for this at some point.
      var focusEl = FocusStack.pop()
      if (modal.dialog || modal.menu) focusEl.focus()

      // If it's the last modal open, restore non-modal state
      if (!Stack.length) {
        Event.off(document, 'keydown', keyPress)
        document.body.classList.remove('modal-active')
        document.body.dataset.modal = ''
      } else {
        document.body.dataset.modal = activeModal().el.id
      }
    })

    return modal
  }
}

// Add events to modal
function on (selector, event, callback) {
  var modal = getModal(selector)
  if (modal && modal.events[event]) {
    modal.events[event].add(callback)
    return modal
  }
}

function close (modal) {
  // Did they specify a modal to close?
  if (typeof modal != 'undefined') {
    modal = getModal(modal)
    if (modal) modal.close()

  // No? Assume current modal should be closed
  } else {
    modal = activeModal()
    if (modal) modal.close()
  }

  return modal
}

function open (modal) {
  modal = getModal(modal)
  if (modal) modal.open()

  return modal
}

// Clicks which occur when a modal is active
function activeClick (event) {
  var modal = activeModal()

  // Stop the click if it happened outside the modal or panel control
  if (!isWithinModal(event.target)) {
    event.stopImmediatePropagation()
    event.preventDefault()

    if (modal.panel) modal.close()
  }
}

function focused(event) {
  var modal = activeModal()

  // If a modal panel, close when focus occurs outside.
  if (document.activeElement && !isWithinModal(document.activeElement)) {
    event.stopImmediatePropagation()
    if (modal.panel) modal.close()
    else event.preventDefault()
  }
}

function isWithinModal(el) {
  var modal = activeModal(),
      // Did the click happen in side the modal
      modalClick = el.closest('[data-modal-id="'+modal.id+'"]'),
      // Or in a modal safe control?
      panelClick = modal.panel && !!el.closest('[data-for-modal="'+modal.id+'"]')

  return modalClick || panelClick
}

// Use the stack position to determine the current modal
function activeModal () {
  return Stack[ Stack.length - 1 ]
}

function activePanel () {
  return activeModal().querySelector('[aria-modal=true][aria-expanded=true]')
}

function isElement (item) {
  return item.constructor.toString().search(/HTML.+Element/) > -1
}

// Returns the modal object by id or selector
function getModal (search) {

  // Don't accept garbage input
  if (!search) return

  // Was a modal object passed in? Just return it
  if (search._modal) return search

  // Is it already registered? (passed an id)
  if (Modals[ search ]) return Modals[ search ]

  // Try modal triggers
  if (isElement(search)) {
    // Is it a modal trigger?
    if (search.dataset.forModal) return Modals[ search.dataset.forModal ]

    // Is it the child of a modal trigger?
    var parent = search.closest('[data-for-modal]')
    if (parent) return Modals[ parent.dataset.forModal ]
  }

  if (typeof search === 'string') {

    // Well, let's hope it's a DOM selector
    var modal = document.querySelector(search)

    // If it is woo hoo
    if (modal && modal.dataset.modalId) {
      return Modals[ modal.dataset.modalId ]
    }
  }
}

function init() {
  var types = ['panel', 'menu', 'dialog']

  // Search for three types of modals
  types.forEach(function(type) {
    toolbox.each(document.querySelectorAll('[data-show-'+type+']:not([data-for-modal])'), function(triggerEl) {
      var modalEl

      // Get selector
      var selector = triggerEl.getAttribute('data-show-'+type)
      if (selector) modalEl = document.querySelector(selector)

      if (modalEl) {
        var modal = Modal.new(modalEl, type, triggerEl)

        // Assign the trigger to this modalId for easy reference later
        triggerEl.dataset.forModal = modal.id
      }
    })
  })

  if (!document.querySelector('#modal-toggle-style')) {
    document.body.insertAdjacentHTML('afterbegin', style)
  }
}

function triggerEvent (event) {
  var modal = getModal(event.currentTarget)
  var el = event.currentTarget

  if (el.tagName == 'INPUT') {

    // Don't open when input is focused
    // if (event.type == 'focusin' && el.value == '') return

    // Do not trigger open if input event clears input value
    if (event.type == 'input' && el.value == '') return
  }

  // Don't follow links used to trigger modals
  if (el.tagName.toLowerCase() == 'a' && el.href == '#') {
    event.preventDefault()
  }


  open(modal)

}

function keyPress (event) {
  var activeEl = document.activeElement

  // ESC was pressed
  if (Event.key.isPressed("esc")) {
    // A way to stop clearing the input value
    //if (activeEl.tagName == 'INPUT' && activeEl.value != '') {
      //event.preventDefault()
    //}
    return close()
  }

  if (activeModal() && Event.key.isPressed("tab") && !Event.key.shift) {
    focusNext(event)
  }
}

function focusNext(event) {
  var modalEl     = activeModal().el,
      focusable   = modalEl.querySelectorAll('input:not([type=hidden]), textarea, select, button, [tabindex]:not([tabindex="-1"])'),
      focusable   = Array.prototype.filter.call(focusable, function(el) {
        return !el.closest('[hidden], [aria-hidden="true"]')
      })

  // If focused on the last focusable element, tab to the first element.
}

function setup () {
  Event.change(init)

  Event.ready(function() {

    // Check all body clicks to see if they should be permitted
    Event.on(document, 'click', 'body.modal-active', activeClick, { useCapture: true })

    // Focus outisde of modal (if a panel) should close modal
    Event.on(document, 'focus', '.modal-active', focused, { useCapture: true })

    // Trigger modals on input events (like a search results panel that waits until typing to show)
    Event.on(document, 'input', 'input[data-for-modal], [data-for-modal] input', Event.debounce(triggerEvent, 50, { leading: true }))

    // Trigger modals on keyup events in inputs (like a search results panel that waits until typing to show)
    Event.on(document, 'click focusin', 'input[data-for-modal], [data-for-modal] input', triggerEvent)

    // Trigger modals on click events for anchors and buttons
    Event.on(document, 'click', 'a[data-for-modal], button[data-for-modal], [data-for-modal] a, [data-for-modal] button', triggerEvent)

    Event.keyOn('esc', close)
  })

  Event.beforeVisit(function(){ close() })
  Event.beforeRender(teardown)
}

function teardown() {
  Event.off(document, 'keydown', keyPress)
  Modals = []
  Stack = []
  FocusStack = []
}

module.exports = {
  new: Modal.new,
  setup: setup,
  modals: Modals,
  init: init,
  get: getModal,
  active: activeModal,
  on: on,
  isElement: isElement
}
