var toolbox      = require('@spark-engine/toolbox'),
    Event        = require('@spark-engine/event'),
    Stacks       = []

var Stack = {
  new: function(root) {

    if (!isElement(root)) root = document.querySelector(root)

    // Don't add a new stack if no element found or if stack already exists
    if (!isElement(root) || root.dataset.stackId) return

    // Set up stack by searching DOM node under it.
    // The first child found will be set up as a default
    // After that each child will be added as a stack push

    var stack = {
      root: root,
      panels: [],
      focus: [],
      current: null,

      add: function (panelEl, name) {
        panelEl.dataset.stackIndex = stack.panels.length

        hideEl(panelEl)

        stack.panels.push(panelEl)
      },

      // Reset stack to first panel
      reset: function () {
        // Already reset? We're done.
        if (stack.current == 0) return

        // If hidden or first is empty, hide all
        if (stack.root.offsetParent == null) {
          stack.focus = [] // Clear focus stack

          // Hide and reset scroll on all panels
          stack.hidePanels()

          // Show first panel
          if (isEmpty(stack.panels[0])) {
            stack.current = null
          } else {
            stack.current = 0
          }
          showEl(stack.panels[0])
          stack.updateWatchers()
        }

        // If not hidden, dismiss current panel and reset to first panel
        else stack.show(0)
      },

      // Show a specific panel, hiding all others
      show: function (name) {
        var panel = stack.findPanel(name)
        if (!panel) return // This panel doesn't exist

        if (isEmpty(panel)) {
          stack.current = null
          stack.hidePanels()
          stack.updateWatchers()
          return
        }

        var current = stack.currentPanel()
        if (!current) return stack.enter(panel) // the stack isn't currently active

        // If a panel isn't the current panel
        if (panel != current) {

          // Dismiss it with the callback load in the next panel
          return stack.dismiss(stack.direction(panel), function() {
            stack.enter(panel)
          })

        }
      },

      enter: function (el) {
        var direction = stack.direction(el)
        el.dataset.direction = direction
        stack.current = Number(el.dataset.stackIndex)

        Event.afterAnimation(el, function() {
          el.dataset.direction = ''
          el.classList.remove('enter')
        }, 50)

        el.classList.add('enter')
        showEl(el)

        stack.updateWatchers()

        // When going forward remove focus from elements in the previous panel
        //if (direction == 'forward') {
          //if (toolbox.childOf(document.activeElement, stack.root)) {
            //document.activeElement.blur()
            //if (navigated) {
              //var navEl = el.querySelector('[data-stack-nav]')
              //if (navEl) navEl.focus()
            //}
          //}
        //}

        // When reversing, focus on the previously focused element
        if (direction == 'reverse' && stack.focus.length > 0) {
          stack.focus.pop().focus()
        }
      },

      dismiss: function (direction, callback) {

        var el = stack.currentPanel()

        if (direction == 'forward')
          stack.focus.push(document.activeElement)

        el.classList.remove('enter')
        el.classList.add('exit')
        el.dataset.direction = direction

        Event.afterAnimation(el, function() {
          // Ensure panel is scrolled to top
          if (direction == 'reverse') el.scrollTop = 0

          hideEl(el)
          el.classList.remove('exit')
          el.dataset.direction = ''

          if (typeof callback === 'function') callback()

        }, 50)

      },

      hidePanels: function() {
        stack.panels.forEach(function(el) {
          el.scrollTop = 0
          hideEl(el)
        })
      },

      lastFocus: function() {
        return stack.focus[ stack.focus.length -1 ]
      },

      // Find a panel by name or index
      findPanel: function (name) {
        if (isElement(name))
          return name
        else
          return root.querySelector('[data-stack="'+name+'"], [data-stack-index="'+name+'"]')
      },

      // Walk backwards in the stack, showing the panel previous to the current panel
      back: function () {
        stack.show(stack.panels[ stack.current - 1 ])
      },

      // add aria-hidden=false to the first hidden element
      next: function () {
        stack.show(stack.panels[ stack.current + 1 ])
      },

      direction: function(panel) {
        var index = stack.panelIndex(panel)

        if (index == stack.current || stack.current == null) return 'none'
        else return (stack.current < index) ? 'forward' : 'reverse'

      },

      currentPanel: function (name) {
        if (typeof stack.current == 'undefined') return

        if (name) {
          return stack.panels[ stack.current ] == stack.findPanel(name)
        }
        return stack.panels[ stack.current ]
      },

      panelIndex: function (panel) {
        return Number(panel.dataset.stackIndex)
      },

      updateWatchers: function () {
        toolbox.each(stack.watchers, function(el) {
          if (stack.current == null)
            el.removeAttribute('data-active-stack')
          else
            el.dataset.activeStack = stack.currentPanel().dataset.stack || stack.currentPanel().dataset.stackIndex
        })
      }
    }

    toolbox.each(root.children, stack.add)

    if (root.id) stack.watchers = document.querySelectorAll('[data-watch-stack="#'+root.id+'"]')

    // Only show the first panel if it has content.
    // If it doesn't, assume async and manual panel showing.
    if (!isEmpty(stack.panels[0])) stack.show(0)

    // Add a queryable stack id
    root.dataset.stackId = Stacks.length

    Stacks.push(stack)

    return stack
  }
}

function showEl(el) {
  el.removeAttribute('hidden')
  el.setAttribute('aria-hidden', false)
}

function isEmpty(el) {
  return el.childElementCount == 0
}

function hideEl(el) {
  el.setAttribute('aria-hidden', true)
  el.setAttribute('hidden', true)
}

function isElement(item) {
  return item.constructor.toString().search(/HTML.+Element/) > -1
}

function navClick(event) {
  var el = event.currentTarget
  var panel = el.dataset.stackNav
  var stack = getStack(el.closest('[data-stack="root"]'))
  if (!stack) return

  if (panel == 'next') stack.next()
  else if (panel == 'back') stack.back()
  else if (stack.findPanel(panel)) {
    stack.show(panel, true)
  }
}

function navKey(event) {
  // Find the active panel in the stack where the navigation event should be triggered.
  var panel = event.currentTarget.querySelector('[data-stack-index][aria-hidden="false"]')
  if (!panel) { return }

  if (Event.key.isPressed("left")) {
    var back = panel.querySelector('.stack-nav-back')
    if (back) {
      Event.fire(back, 'click')
      event.preventDefault()
    }
  }
  if (Event.key.isPressed("right")) {
    var next = panel.querySelector('.stack-nav-next')
    if (next) {
      Event.fire(next, 'click')
      event.preventDefault()
    }
  }
}

function prevPanel (event) {
  var stack = getStack(event.target.closest('[data-stack="root"]'))
  stack.back()
  event.preventDefault()
}

function getStack (search) {

  if (isElement(search)) {
    if (search.dataset.stackId) return Stacks[ search.dataset.stackId ]
  }

  if (typeof search === 'string') {

    // Well, let's hope it's a DOM selector
    var stack = document.querySelector(search)

    // If it is woo hoo
    if (stack && stack.dataset.stackId) {
      return Stacks[ stack.dataset.stackId ]
    }
  }
}

function setup () {
  Event.change(function() {
    toolbox.each(document.querySelectorAll('[data-stack="root"]'), Stack.new)
  })

  // Localise clicks to a stack root
  // Future: consider expanding this to allow controls to live anywhere and point to a stack
  Event.ready(function() {
    Event.on(document, 'click', '[data-stack="root"] [data-stack-nav]', navClick)
    Event.on(document, 'keydown', '[data-stack="root"]', navKey)
  })

  Event.beforeRender(teardown)
}

function teardown() {
  Stacks = []
}

module.exports = {
  new: Stack.new,
  setup: setup,
  get: getStack,
  stacks: Stacks
}
