// Purpose: Toggles expansion of tree nav and stores cookies to retain state
//
// For Cookies to be set:
// - Trees must have a [data-tree='tree-key']
// - Nodes are named by [data-node='node-name']
//
// Cookies are stored as tree-key => { node-name: true }
// node boolean matches aria-expanded state
//

var toolbox     = require('@spark-engine/toolbox'),
    cookie      = require('../../_cookie'),
    Event       = require('@spark-engine/event')

function setCookie(node) {
  if (node.dataset.node) {
    var tree = node.closest('[data-tree]')
    if (tree) {
      data = {}
      data[ node.dataset.node ] = node.getAttribute('aria-expanded') == 'true'
      cookie.updateJson(tree.dataset.tree, data)
    }
  }
}

function clickTrigger(event) {
  toggleNode(event.currentTarget)
}

function clickLink(event) {
  setActive(event.currentTarget)
}

// Switch currently active tree to clicked items parent tree
function setActive(item) {
  toggleActive(item, true)
}

function setInactive(item) {
  toggleActive(item, false)
}

function toggleActive(item, active) {
  var parentNode = item.closest('[role="treeitem"][aria-expanded]')
  if (parentNode) { 
    parentNode.classList.toggle('active', active) 
    if (active) { toggleExpansion(parentNode, active) }
  }
}

// Expand or collapse the current tree node
function toggleNode(target) {
  var parent   = target.closest('[aria-expanded]'),
      expanded = parent.getAttribute('aria-expanded') == 'true'

  // Add a classname to indicate that it was manually triggered
  // This allows for animating interactions, but not other attribute changes
  parent.classList.add('triggered')
  toggleExpansion(parent, !expanded)

  setCookie(parent)
}

// Set aria-expanded and toggle tabindex values based on expansion
function toggleExpansion(el, expanded) {
  el.setAttribute('aria-expanded', expanded)
  setupItemTabIndex(el)
}

// Sets tree nav state based off of cookies
function restoreNav() {
  toolbox.each(document.querySelectorAll('[data-tree]'), function(tree) {
    var key = tree.dataset.tree
    if (key) {
      var data = cookie.getJson(key)

      Object.keys(data).forEach(function(node) {
        var nodeEl = tree.querySelector('[data-node='+node+']'),
            expanded = data[node] == 'true'
        // Use cookies to expand.
        if (nodeEl)  {
          nodeEl.setAttribute('aria-expanded', expanded)
          toggleExpansion(nodeEl, expanded)
        }
      })
    }
  })
}

// Enables keyboard navigation of a nav tree
//
// `space`       - toggles tree node triggers
// `right arrow` - opens nodes and navigates inward
// `left arrow`  - closes nodes and navigates outward
// `up arrow`    - moves to previous visible element
// `down arrow`  - moves to next visible element
//
function navigateTree(event) {
  var item = document.activeElement
  if (!item) return

  // Find the parent tree item
  var tree = item.closest('[role="tree"]')
  if (!tree) return

  // Is the item a trigger which opens nodes?
  if (item.matches('.tree-trigger')) {
    var expanded = item.parentElement.getAttribute('aria-expanded') != 'false'

    // Space toggles node which trigger points to
    if (Event.key.isPressed('space')) {
      event.preventDefault()
      return toggleNode(item)
    }

    // Expand or move to the first item in the node
    if (Event.key.isPressed('right')) {
      event.preventDefault()
      if (expanded) return focus(getNextItem(tree, item))
      else return toggleNode(item)
    }

    // Collapse or move up the parent node
    if (Event.key.isPressed('left')) {
      event.preventDefault()
      if (expanded) return toggleNode(item)
      else return focus(getUpwardItem(tree, item))
    }
  }

  // Move to next visible item in the tree
  if (Event.key.isPressed('down')) {
    event.preventDefault()
    focus(getNextItem(tree, item))
  }

  // Move to previous visible item in the tree
  else if (Event.key.isPressed('up')) {
    event.preventDefault()
    focus(getPreviousItem(tree, item))
  }

  // Move outward in tree
  else if (Event.key.isPressed('left')) {
    event.preventDefault()
    focus(getUpwardItem(tree, item))
  }
}

// Focus on element (if able)
function focus(el) {
  if (el) el.focus()
}

// Find next focusable item
function getNextItem(tree, item) {
  items = getFocusableItems(tree)
  return items[ items.indexOf(item) + 1]
}

// Find previous focusable item
function getPreviousItem(tree, item) {
  items = getFocusableItems(tree)
  return items[ items.indexOf(item) - 1]
}

// Walk up the tree to the tree trigger
function getUpwardItem(tree, item) {
  var upward = item.parentElement.closest('[aria-expanded]')
  if (upward) return upward.querySelector('.tree-trigger')
}

// Find all focusable items in a nav tree
function getFocusableItems(tree) {
  // Filter out items in collapsed nodes
  return Array.prototype.filter.call(tree.querySelectorAll('a, [tabindex]'), function(el) {
    return !el.closest('[aria-expanded="false"] [role="group"]')
  })
}

// Ensure hidden items are not focasable
function setupItemTabIndex(tree) {
  toolbox.each(tree.querySelectorAll('a, [tabindex]'), function(el) {
    if (el.closest('[aria-expanded="false"] [role="group"]')) {
      el.setAttribute('tabindex', -1)
    } else {
      // Don't set a tabindex for anchor elements
      // They already participate in tabindex and tabindex=0
      // will change their click behavior
      if (el.matches('a')) {
        el.removeAttribute('tabindex')
      } else {
        el.setAttribute('tabindex', 0)
      }
    }
  })
}

function setup() {
  Event.ready(function() {
    // The first child of a node is the trigger
    Event.on(document, 'click', '.tree-trigger', clickTrigger)

    //Event.on(document, 'click', '[role=tree] .nav-item[href]:not([href="#"])', clickLink)
    Event.on(document, 'keydown', '[role="tree"]', navigateTree)

    // Set states according to cookies
    // restoreNav()
  })

  Event.change(function() {
    toolbox.each(document.querySelectorAll('[role="tree"]'), function(el) {
      setupItemTabIndex(el)
    })
  })
}

module.exports = {
  setup: setup,
  setActive: setActive,
  setInactive: setInactive
}
