var toolbox          = require('@spark-engine/toolbox'),
    Event            = require('@spark-engine/event'),
    stickyClass      = 'ui-sticky-panel',
    stickyStateClass = 'is-sticky',
    stickySpacer     = 'ui-sticky-panel-spacer',
    scrollPanels     = []


function setup() {
  scrollPanels.push({ panel: window, els: [] })
  Event.scroll.start(startScrolling)
  Event.scroll(whileScrolling)

  Event.change(function(){
    toolbox.each(document.querySelectorAll('[data-sticky]:not([data-sticky-start])'), setupSticky)
  })

  Event.resize(function(event) {
    // Set new start points due to reflow
    toolbox.each(document.querySelectorAll('[data-sticky]'), setStart)
    // Reset constraints when page is resized
    toolbox.each(document.querySelectorAll('.'+stickyClass), stick)
  })
}

function setupSticky(el) {
  setupPanels(el)
  appendSpacer(el)
  setStart(el)
}

function startScrolling(event) {
  getEventPanel(event.currentTarget).els.forEach(setStart)
}

function whileScrolling(event) {
  var panel    = event.currentTarget,
      position = panel == window ? panel.scrollY : panel.scrollTop
  getEventPanel(event.currentTarget).els.forEach(function(el) {
    var hasStickClass = el.classList.contains(stickyClass)
    var stickable = !isFixed(el) && !hasStickClass
    if (Number(el.dataset.stickyStart) < position) {
      if (stickable) stick(el)
    } else if (hasStickClass) unstick(el)

  })
}

// Set an elements style properties to fixed position and constrain its dimensions.
function stick(el) {
  // Don't calculate spacer height from a fixed position element.
  if(!isFixed(el)) {
    setSpacerStyle(el)
    el.classList.add(stickyClass)
    el.classList.remove(stickyStateClass)
  }

  // If position is fixed (by adding class) add calculated styles
  if (isFixed(el)) {
    var panel = getPanel(el)
    // Fix just below top of panel
    el.style.top   = window.getComputedStyle(panel).marginTop
    el.classList.add(stickyStateClass)

    // Constrain to spacer (retaining natural width) or full panel width
    var constraint = el.hasAttribute('data-sticky-constrain') ? getSpacer(el) : getPanel(el)

    el.style.left  = getOffset(constraint).left + 'px'
    el.style.width = constraint.clientWidth + 'px'
  }
}

function unstick(el) {
  clearStyle(el)
  el.classList.remove(stickyClass, stickyStateClass)
}

function clearStyle(el) {
  el.style.left = ''
  el.style.top  = ''
  el.style.width = ''
}

// Spacer keeps content from jumping when element's position is fixed
function setSpacerStyle(el) {
  var spacer = el.nextSibling
  spacer.style.height = el.clientHeight + 'px'
  spacer.style.margin = window.getComputedStyle(el).margin
}

function appendSpacer(el) {
  var spacer = getSpacer(el)
  if(!spacer || !spacer.classList.contains(stickySpacer)) {
    el.insertAdjacentHTML('afterend', "<div class='"+stickySpacer+"'></div>")
  }
}

function getSpacer(el) {
  return el.nextElementSibling
}

// Associate sticky elements with their panels, adding panels if necessary
function setupPanels(el) {
  var panel,
      panelId = el.dataset.scrollPanelId

  if (!panelId) {
    panelId = getPanelId(el)

    el.dataset.scrollPanelId = panelId
    scrollPanels[panelId].els.push(el)
  }
}

// Add a panel to the scrollPanels object
function addPanel(panel) {
  scrollPanels.push({
    panel: panel,
    els: []
  })
  panel.dataset.scrollPanelId = scrollPanels.length - 1
  addManager(panel)
}

// Add a new scroll event manager for a panel
function addManager(target) {
  var scroll = Event.eventManager.new('scroll', { target: target })
  scroll.start(startScrolling)
  scroll(whileScrolling)
}

// Set the y coordinate when element should become sticky while scrolling
function setStart(el) {
  if(!isFixed(el)) {
    el.dataset.stickyStart = getOffset(el).top - getPanelOffset(el)
  }
}

function getPanelOffset(el) {
  var panel = getPanel(el)
  return getOffset(panel).top + panel.scrollTop
}

// Get Panel element referred to by an element with a panel id
// If scroll panel is window, return body for DOM measurments
function getPanel(el) {
  var panel = scrollPanels[el.dataset.scrollPanelId].panel
  return panel == window ? document.body : panel
}

// Return panel object deriving panel id from event target
function getEventPanel(el) {
  var id = el == window ? 0 : Number(el.dataset.scrollPanelId)
  return scrollPanels[id]
}

// Return the scroll panel for an element, either:
// 1. Return the panel it refers to with its data sticky selector.
// 2. Or walk up the DOM tree to find the closest `data-scroll` element.
function getPanelId(el) {
  // Either retrieve a custom panel, or find the next closest
  var panel = el.parentElement.closest('[data-scroll]')

  if (panel) {
    // Hasn't been set up yet
    if (!panel.dataset.scrollPanelId) addPanel(panel)

    return Number(panel.dataset.scrollPanelId)
  } else {
    // If no scroll panels are found, default to window scoll
    return 0
  }
}


// Is actually fixed? Must check computed style to support media query changes
function isFixed(el) {
  return window.getComputedStyle(el).position === 'fixed'
}

// Get offset including offsets of parents
function getOffset(el) {
  var _x = 0;
  var _y = 0;
  while(el && !isNaN(el.offsetLeft) && !isNaN(el.offsetTop)) {
    _x += el.offsetLeft - el.scrollLeft;
    _y += el.offsetTop - el.scrollTop;
    el = el.offsetParent;
  }
  return { top: _y, left: _x };
}

module.exports = {
  setup: setup,
  panels: scrollPanels,
  getOffset: getOffset,
  getPanel: getPanel,
  getPanelOffset: getPanelOffset
}
