module Spark
  class Placeholder < Spark::Component::Base
    SIZE = { sm: 24, md: 32, lg: 44, xl: 84 }
    THEME = {
      default: {
        text: "#ffffff",
        fill: "#78858A"
      },
      gradient: {
        gradient: %w[#616c70 #959fa3],
        fill: "#78858A"
      },
      wisteria: {
        gradient: %w[#A8A2EC #5853AE]
      }
    }

    attribute :alt, :text, :circle, :font_size, :width, :height, :icon, size: :md, theme: :default
    attribute remote: false

    validates_attr :theme, choices: THEME.keys

    def initialize(*)
      super

      aria.add(hidden: true)
      classname.base = "spark-icon-placeholder-wrapper" unless @remote

      @height ||= @width ||= SIZE[@size]
      tag_attrs.add(attr_hash(:width, :height))
      aria.add attr_hash(:alt)

      return if @remote

      raise no_icon_found if @icon && !Rails.env.production? && !Esvg.find_symbol(@icon)

      classname.add("spark-icon-placeholder")
      tag_attrs.add(svg_attrs)
    end

    def options
      return @options if @options

      @options ||= attr_hash(:width, :height, :alt, :text)
      @options[:class] = shape_class
      @options
    end

    private

    def remote_image_url
      fill       = background_fill.sub('#','')
      text_color = text_fill.sub('#','')
      "https://via.placeholder.com/#{@width * 2}x#{@height * 2}/#{fill}/#{text_color}?text=#{@text || "%20"}"
    end

    def remote_attrs
      classname.add(shape_class)
      tag_attrs
    end

    def shape_class
      @circle ? "spark-circle-crop" : "spark-roundrect-crop"
    end

    def svg_attrs
      {
        class: "ui-placeholder",
        aria: { presentation: :true, label: @alt },
        viewBox: "0 0 #{@width} #{@height}",
        version: "1.1",
        xmlns: "http://www.w3.org/2000/svg"
      }
    end

    def icon_attrs
      svg_scale = 0.63
      center_scale = (1 - svg_scale) / 2
      # make size proportional to container
      {
        color: text_fill,
        width: (@width * svg_scale).round,
        height: (@height * svg_scale).round,
        x: (@width * center_scale).round,
        y: (@height * center_scale).round
      }
    end

    def text_attrs
      {
        x: @width/2,
        y: @height/2,
        fill: text_fill,
        "font-size" => @font_size || "#{@width/3}px",
        "text-anchor" => :middle,
        "dominant-baseline" => :central,
        "alignment-baseline" => :central
      }
    end

    def theme
      THEME[@theme]
    end

    def text_fill
      theme[:text] || THEME[:default][:text]
    end

    def background_fill
      theme[:fill] || theme[:gradient].first
    end

    def background
      if gradient?
        "url(##{gradient_id})"
      else
        background_fill
      end
    end

    def gradient_id
      "placeholder-gradient-#{@theme}"
    end

    def gradient
      theme[:gradient]
    end

    def gradient?
      !!gradient
    end

    def no_icon_found
      %(Cannot find icon "#{@icon}". A list of available icons can be found at http://spark.fleetio.com/resources/icons)
    end
  end
end
