module Spark
  module InputHelper

    def input_tag(type, name, value, options=nil, &block)
      type.to_sym! if type.is_a?( String )

      if value.is_a? Hash
        options = value
        value = nil
      end

      options = Spark::Helpers.set_type_defaults( type, options )
      options = Spark::Helpers.set_data_keys( options, 'default' )
      options = Spark::Helpers.set_aria_keys( options )

      icon = Esvg.use( icon ) if icon = options.delete('icon')
      icon_after = Esvg.use( icon_after ) if icon_after = options.delete('icon_after')

      options['data']['unique-id'] = "input-#{SecureRandom.hex(8)}"
      options['autocomplete'] ||= 'off' if type != :textarea

      label_options = { 
        data: { 'input-type' => type },
        class: [ label_class( type ), options.delete( 'class' ) ]
      }

      # Move modal options to label
      %w(show-panel show-dialog show-menu).each do |opt|
        if val = options['data'].delete(opt)
          label_options[:data][opt] = val
        end
      end

      # Add a placeholder element unless textarea
      if type != :textarea && placeholder = options['placeholder']
        placeholder = content_tag(:span, class: 'input-placeholder-text') { placeholder }
        label_options = add_class label_options, 'placeholder-label'
      end

      if label_description = options.delete('description')
        options['aria-describedby'] = options['data']['unique-id'] + '-description'
        label_description = content_tag(:span, id: options['aria-describedby'], class: 'label-description') { label_description }
        label_options = add_class label_options, 'has-description'
      end

      if options['label'] && label_text = options.delete('label')
        options['aria-labelledby'] = options['data']['unique-id'] + '-label'

        label_text = content_tag(:span, class: 'input-label-text') {
          content_tag(:span, id: options['aria-labelledby']){ label_text }
        }
      end

      # Be sure the input has a label if there is no label text
      options['aria-label'] = options['placeholder'] if placeholder && !label_text

      input = base_input_tag( name, value, options, type, &block )

      content_tag(:div, label_options) do
        concat( label_text )
        concat( label_description )

        concat content_tag( :span, class: 'input-wrapper' ) {
          if type == :select
            concat( input )
            #concat placeholder
          else
            concat content_tag( :span, class: 'input-icon' ) { icon } if icon

            concat content_tag( :span, class: 'input-container' ) {
              concat( input )
              #concat placeholder
            }

            if type == :search 
              concat button_tag( type: 'button', class: 'clear-search', "aria-label" => 'Clear search', disabled: !value, "data-clear-input" => "[data-unique-id=#{options['data']['unique-id']}]" ) { 'x' }
            end
            concat content_tag( :span, class: 'input-icon after' ) { icon_after } if icon_after

            if block_given? && type != :textarea
              html = capture(&block)

              if html.match(/<select.*?>/)
                selected = html.match(/<option.*?selected.*?>(.+?)<\/option>/m) || html.match(/<option.*?>(.+?)<\/option/m)
                html.sub!(/<select/, "<select data-sync-selected='[data-sync=#{options['data']['unique-id']}]' ")
                html << "<span class='select-text' data-sync='#{options['data']['unique-id']}'>#{selected[1]}</span>"
              end

              concat content_tag( :span, class: 'input-extra' ) { html.html_safe }
            end
          end
        }
      end

    end

    def base_input_tag( name, value, options, type, &block )
      case type
      when :range
        range_input_tag( name, options )
      when :select
        value = value.html_safe if value
        select_tag( name, value, options )
      when :textarea
        value ||= capture(&block).html_safe if block_given?
        options[:class] = "#{options[:class]} empty".strip if value.nil?
        text_area_tag( name, value, options )
      else
        options[:class] = "#{options[:class]} empty".strip if value.nil?
        text_field_tag( name, value, options )
      end

    end

    def range_input_tag(name, options)
      options.merge!({ type: "range", })

      tag :input, options
    end

    def label_class( type )
      type = case type
      when :tel, :password, :number, :url, :email, :search
        "text"
      when :checkbox, :radio
        "tick"
      else
        type.to_s
      end

      "#{type}-input-label"
    end

    def switch_input_tag(name, checked = false, options = {})
      type = :checkbox

      if checked.is_a? Hash
        options = checked
        checked = false
      end

      options = add_class( options, [ label_class( type ), 'check-switch switch-label'] )

      label_options = { 
        data: { 
          'input-type' => type,
          'input' => 'checkbox'
        },
        class: options.delete('class')
      }

      if label_text = options.delete('label')
        label_text = content_tag(:span, class: 'label-text') { label_text }
      end

      content_tag(:label, label_options ) do
        concat tag :input, name: name, type: :hidden, value: false
        concat label_text
        concat check_box_tag(name, true, checked, options)

        concat content_tag(:span, class: 'check-switch-panel') {
          concat content_tag(:span, class: 'check-switch-tick') { '' }
        }

        concat content_tag(:span, class: 'check-switch-label') { 'Enable' }
      end

    end

  end
end
