module Spark
  module MenuHelper 
    class Menu < Spark::Helper
      DEFAULTS = {
        tag: :div,
        role: :menu,
        base_class: 'menu',
        header_tag: :h4,
        group_tag: :ul,
        item_tag: :li
      }

      def initialize( title = nil, options = {} )
        options, title = title, nil if title.is_a? Hash

        @options = add_class( options, config[:base_class] )
        @options['aria-label'] = title if title
      end

      def display( body )
        @options.merge!({ role: config[:role] })
        content_tag( config[:tag], @options ) do
          concat( body )
        end
      end

      def header( text = nil, options = {}, &block )
        options, text = text, nil if text.is_a?( Hash )
        options = add_class( options, config[:base_class] + "-header" )

        content_tag( :header, options ) {
          concat( content_tag( config[:header_tag], text, class: config[:base_class] + '-title' ) ) if text
          concat( capture( &block ).html_safe ) if block_given?
        }
      end

      def link( content=nil, url=nil, options=nil, &block )
        content, url, options = parse_item_args( content, url, options, &block)
        content, options = set_content( content, options )
        options = add_class( options, config[:base_class] + "-item" )

        options['role'] ||= 'menuitem'

        link_to( url, options, &content )
      end

      def group( title=nil, options={}, &block )
        options = add_class( options, config[:base_class] + "-group" )
        options['role'] = 'group'

        content = capture( &block )
        return if content.blank? ||
                ( options.include?('if') && !options.delete('if') ) ||
                ( options.include?('unless') && !!options.delete('unless') )                     

        concat header( title ) if title
        content_tag( config[:group_tag], options, &block )
      end

      def item( content=nil, url=nil, options=nil, &block )
        content, url, options = parse_item_args( content, url, options, &block)

        return if ( options.include?('if') && !options.delete('if') ) ||
                  ( options.include?('unless') && !!options.delete('unless') )

        if url
          content_tag( config[:item_tag], role: 'none', class: config[:base_class] + "-item-wrapper" ) do
            link( url, options ) { content }
          end
        else
          content, options = set_content( content, options )
          options = add_class( options, config[:base_class] + "-item" )
          content_tag( config[:item_tag], options, &content )
        end
      end

      private

      # add icons, wrap content, strip bad option keys
      def set_content( content, options )
        icon = options.delete('icon')
        after_icon = options.delete('after-icon')

        icon_options = {
          class: config[:base_class] + "-item-icon"
        }.merge( options.delete('icon-options') || {} )

        after_icon_options = {
          class: config[:base_class] + "-item-after-icon"
        }.merge( options.delete('after-icon-options') || {} )

        inner_content = lambda {
          concat use_svg( icon, icon_options ) if icon
          concat content_tag( :div, class: config[:base_class] + "-item-content" ) { content }
          concat use_svg( after_icon, after_icon_options ) if after_icon
        }

        [inner_content, options]
      end

      def config(options=nil)
        @_config = DEFAULTS.merge(options) if options
        @_config ||= DEFAULTS
      end

      # Simplify setting up options and content
      def parse_item_args( content=nil, url=nil, options=nil, &block )
        # If block passed, shift args so content = block
        options, url, content = url, content, capture( &block ) if block_given?
        # If url is a hash, url wasn't passed, set to nil, an shift args
        options, url = url, nil if url.is_a?(Hash)
        options ||= {}

        options = Spark::Helpers.set_data_keys( options )

        [content, url, options]
      end
    end
  end
end
