module Spark
  module NavMenuHelper
    def parse_cookie( key )
      JSON.parse( cookies[ key ] ) if cookies[ key ].present?
    end

    class NavMenu < Spark::MenuHelper::Menu

      def initialize( title = nil, options = {} )
        config({
          tag: :nav,
          base_class: 'nav-menu'
        })
        super( title, options )
      end

      def more( content=nil, url=nil, options=nil, &block )
        content, url, options = parse_item_args( content, url, options, &block)
        options['aria-haspopup'] = 'true'
        options = add_class( options, config[:base_class] + "-more" )
        url ||= '#'

        item( content, url, options )
      end
    end

    class TreeNav < Spark::MenuHelper::Menu

      def initialize( title = nil, options = {} )
        config({
          tag: :ul,
          base_class: 'nav-tree',
          role: 'tree'
        })
        super( title, options )
      end

      def item(content, url, options={}, &block)
        options['role'] = 'treeitem'

        super( content, url, options, &block )
      end

      def item_tree( name, options={}, &block )
        content, url, options = parse_item_args( "#", options, &block)

        return if content.blank? ||
                  ( options.include?('if') && !options.delete('if') ) ||
                  ( options.include?('unless') && !!options.delete('unless') )

        node = Spark::Helpers.dasherize( name.strip.downcase ).gsub('-','_')
        active  = !!(options.delete( 'active' ) || content.match(/item-active/))

        options.merge!({
          'role' => 'treeitem',
          'aria-expanded' => active || is_expanded?(node),
          'data-node' => node
        })

        name, options = set_content( name, options )

        classes = [config[:base_class] + "-item-wrapper"]
        classes.push( 'tree-active' ) if !!( options.delete( 'active' ) || content.match(/item-active/))

        options = add_class( options, classes )

        content_tag( config[:item_tag], options ) do
          concat link_to("#", class: config[:base_class] + '-item'){
            concat name.call
            concat use_svg( 'chevron-down', class: 'expand-icon'  )
          }
          concat content_tag( :ul, role: 'group', class: config[:base_class] + '-group' ) { content }
        end
      end

      private

      def _cookie
        @_cookie ||= begin
          parse_cookie( @tree )
        rescue
          nil
        end
      end

      def is_expanded?( item )
        !@tree.nil? && _cookie.present? && !!_cookie[ item ]
      end
    end

    class SidebarNav < TreeNav
      def initialize( title, options = {} )
        @tree = 'sidebar_nav'
        options['data-tree'] = @tree
        super( title, options )
      end

      def item( content, url, options={}, &block )
        options = options.stringify_keys
        new = is_new?( options.delete( 'new' ) )
        beta = is_beta?( options.delete( 'beta' ) )
        active = is_active?( options.delete( 'active' ) )

        classes = []
        classes.push( 'item-beta' )   if beta
        classes.push( 'item-new' )    if new
        classes.push( 'item-active' ) if active
        options = add_class( options, classes )


        super( content, url, options, &block)
      end


      def item_tree( name, options={}, &block )
        options = options.stringify_keys
        content = capture( &block )

        classes = []
        classes.push( 'tree-new' )    if is_new?( options.delete( 'new' ) ) || content.match(/item-new/)
        classes.push( 'tree-beta' )   if is_beta?( options.delete( 'beta' ) ) || content.match(/item-beta/)

        options = add_class( options, classes )

        super( name, options, &block )
      end

      private

      def is_new?( new )
        if new.is_a?(String)
          30.days.ago <= Time.zone.parse( new )
        else
          new
        end
      end

      def is_beta?( beta )
        beta
      end

      def is_active?( active )
        active
      end

    end
  end
end
